#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Symbols;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis.MultiVariate.TimeSeriesPrognosis.Symbolic.Symbols;
using HeuristicLab.Problems.DataAnalysis.Symbolic.Symbols;
using System.Linq;
using HeuristicLab.Problems.DataAnalysis.MultiVariate.TimeSeriesPrognosis.Symbolic.Interfaces;

namespace HeuristicLab.Problems.DataAnalysis.MultiVariate.TimeSeriesPrognosis.Symbolic {
  [StorableClass]
  [Item("SymbolicTimeSeriesPrognosisModel", "Represents a model for time series prognosis.")]
  public class SymbolicTimeSeriesPrognosisModel : NamedItem, IMultiVariateDataAnalysisModel {
    [Storable]
    private ISymbolicTimeSeriesExpressionInterpreter interpreter;
    public ISymbolicTimeSeriesExpressionInterpreter Interpreter {
      get { return interpreter; }
    }
    [Storable]
    private SymbolicExpressionTree tree;
    public SymbolicExpressionTree SymbolicExpressionTree {
      get { return tree; }
    }
    [StorableConstructor]
    protected SymbolicTimeSeriesPrognosisModel(bool deserializing) : base(deserializing) { }
    protected SymbolicTimeSeriesPrognosisModel(SymbolicTimeSeriesPrognosisModel original, Cloner cloner)
      : base(original, cloner) {
        tree = cloner.Clone(original.tree);
        interpreter = cloner.Clone(original.interpreter);
    }
    public SymbolicTimeSeriesPrognosisModel() { }
    public SymbolicTimeSeriesPrognosisModel(ISymbolicTimeSeriesExpressionInterpreter interpreter, SymbolicExpressionTree tree) {
      this.tree = tree;
      this.interpreter = interpreter;

    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SymbolicTimeSeriesPrognosisModel(this, cloner);
    }

    public IEnumerable<double[]> GetEstimatedValues(MultiVariateDataAnalysisProblemData problemData, int start, int end, int horizon) {
      IEnumerable<int> rows = Enumerable.Range(start, end - start);
      IEnumerable<string> targetVariables = from item in problemData.TargetVariables.CheckedItems
                                            select item.Value.Value;
      return interpreter.GetSymbolicExpressionTreeValues(tree, problemData.Dataset, targetVariables, rows, horizon);
    }

    #region IMultiVariateDataAnalysisModel Members

    public IEnumerable<double[]> GetEstimatedValues(MultiVariateDataAnalysisProblemData problemData, int start, int end) {
      return GetEstimatedValues(problemData, start, end, 1);
    }
    #endregion

  }
}